# -*- mode: python; python-indent: 2 -*-
#
# xbrightness-rgbgamma - X11 brightness / gamma adjustment tool
#
# Copyright (C) 2011 kakurasan
#
# Permission is hereby granted, free of charge, to any person obtaining a copy
# of this software and associated documentation files (the "Software"), to deal
# in the Software without restriction, including without limitation the rights
# to use, copy, modify, merge, publish, distribute, sublicense, and/or sell
# copies of the Software, and to permit persons to whom the Software is
# furnished to do so, subject to the following conditions:
#
# The above copyright notice and this permission notice shall be included in
# all copies or substantial portions of the Software.
#
# THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
# IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
# FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
# AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
# LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
# OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN
# THE SOFTWARE.
#
# THANKS:
#  xbrightness by Asher Blum and Jean-Daniel Pauget

from xbrightness_rgbgamma.AppInfo import AppInfo
from xbrightness_rgbgamma.Xpm import Xpm, SingleColorXpm, AlternateXpm
import gtk


class GammaChart (gtk.Image):
  """
  Gamma chart
  """
  def __init__ (self, gamma):
    gtk.Image.__init__ (self)
    (V000, V025, V050, V075, V100) = range (5)
    values = []
    values.append ('{0:02x}'.format (int (round (pow (  0 / 255.0, 1.0 / gamma) * 255))))
    values.append ('{0:02x}'.format (int (round (pow ( 63 / 255.0, 1.0 / gamma) * 255))))
    values.append ('{0:02x}'.format (int (round (pow (127 / 255.0, 1.0 / gamma) * 255))))
    values.append ('{0:02x}'.format (int (round (pow (191 / 255.0, 1.0 / gamma) * 255))))
    values.append ('{0:02x}'.format (int (round (pow (255 / 255.0, 1.0 / gamma) * 255))))
    xpm_side = Xpm.size * 2
    image_side = xpm_side * 12
    pixbufs = []
    # all
    pixbufs.append ((gtk.gdk.pixbuf_new_from_xpm_data (AlternateXpm (values[V000], values[V000], values[V000], values[V050], values[V050], values[V050]).data),
                    gtk.gdk.pixbuf_new_from_xpm_data (SingleColorXpm (values[V025], values[V025], values[V025]).data)))
    pixbufs.append ((gtk.gdk.pixbuf_new_from_xpm_data (AlternateXpm (values[V000], values[V000], values[V000], values[V100], values[V100], values[V100]).data),
                   gtk.gdk.pixbuf_new_from_xpm_data (SingleColorXpm (values[V050], values[V050], values[V050]).data)))
    pixbufs.append ((gtk.gdk.pixbuf_new_from_xpm_data (AlternateXpm (values[V050], values[V050], values[V050], values[V100], values[V100], values[V100]).data),
                    gtk.gdk.pixbuf_new_from_xpm_data (SingleColorXpm (values[V075], values[V075], values[V075]).data)))
    # red
    pixbufs.append ((gtk.gdk.pixbuf_new_from_xpm_data (AlternateXpm (values[V000], values[V000], values[V000], values[V050], values[V000], values[V000]).data),
                    gtk.gdk.pixbuf_new_from_xpm_data (SingleColorXpm (values[V025], values[V000], values[V000]).data)))
    pixbufs.append ((gtk.gdk.pixbuf_new_from_xpm_data (AlternateXpm (values[V000], values[V000], values[V000], values[V100], values[V000], values[V000]).data),
                     gtk.gdk.pixbuf_new_from_xpm_data (SingleColorXpm (values[V050], values[V000], values[V000]).data)))
    pixbufs.append ((gtk.gdk.pixbuf_new_from_xpm_data (AlternateXpm (values[V050], values[V000], values[V000], values[V100], values[V000], values[V000]).data),
                     gtk.gdk.pixbuf_new_from_xpm_data (SingleColorXpm (values[V075], values[V000], values[V000]).data)))
    # green
    pixbufs.append ((gtk.gdk.pixbuf_new_from_xpm_data (AlternateXpm (values[V000], values[V000], values[V000], values[V000], values[V050], values[V000]).data),
                      gtk.gdk.pixbuf_new_from_xpm_data (SingleColorXpm (values[V000], values[V025], values[V000]).data)))
    pixbufs.append ((gtk.gdk.pixbuf_new_from_xpm_data (AlternateXpm (values[V000], values[V000], values[V000], values[V000], values[V100], values[V000]).data),
                       gtk.gdk.pixbuf_new_from_xpm_data (SingleColorXpm (values[V000], values[V050], values[V000]).data)))
    pixbufs.append ((gtk.gdk.pixbuf_new_from_xpm_data (AlternateXpm (values[V000], values[V050], values[V000], values[V000], values[V100], values[V000]).data),
                       gtk.gdk.pixbuf_new_from_xpm_data (SingleColorXpm (values[V000], values[V075], values[V000]).data)))
    # blue
    pixbufs.append ((gtk.gdk.pixbuf_new_from_xpm_data (AlternateXpm (values[V000], values[V000], values[V000], values[V000], values[V000], values[V050]).data),
                      gtk.gdk.pixbuf_new_from_xpm_data (SingleColorXpm (values[V000], values[V000], values[V025]).data)))
    pixbufs.append ((gtk.gdk.pixbuf_new_from_xpm_data (AlternateXpm (values[V000], values[V000], values[V000], values[V000], values[V000], values[V100]).data),
                      gtk.gdk.pixbuf_new_from_xpm_data (SingleColorXpm (values[V000], values[V000], values[V050]).data)))
    pixbufs.append ((gtk.gdk.pixbuf_new_from_xpm_data (AlternateXpm (values[V000], values[V000], values[V050], values[V000], values[V000], values[V100]).data),
                      gtk.gdk.pixbuf_new_from_xpm_data (SingleColorXpm (values[V000], values[V000], values[V075]).data)))
    # output
    pb_out = gtk.gdk.Pixbuf (gtk.gdk.COLORSPACE_RGB, False, 8, image_side, image_side)
    # build
    for (i, (pb_alternate, pb_single)) in enumerate (pixbufs):
      for j in range (3):
        k = j + (i / 3) * 3
        l = i % 3
        if j % 2 == 0:
          pb_alternate.copy_area (0, 0, xpm_side, xpm_side, pb_out, xpm_side * (l * 4), xpm_side * k)
          pb_alternate.copy_area (0, 0, xpm_side, xpm_side, pb_out, xpm_side * (l * 4) + xpm_side * 2, xpm_side * k)
          pb_single.copy_area (0, 0, xpm_side, xpm_side, pb_out, xpm_side * (l * 4) + xpm_side * 1, xpm_side * k)
          pb_single.copy_area (0, 0, xpm_side, xpm_side, pb_out, xpm_side * (l * 4) + xpm_side * 3, xpm_side * k)
        else:
          pb_alternate.copy_area (0, 0, xpm_side, xpm_side, pb_out, xpm_side * (l * 4) + xpm_side, xpm_side * k)
          pb_alternate.copy_area (0, 0, xpm_side, xpm_side, pb_out, xpm_side * (l * 4) + xpm_side * 3, xpm_side * k)
          pb_single.copy_area (0, 0, xpm_side, xpm_side, pb_out, xpm_side * (l * 4) + xpm_side * 2, xpm_side * k)
          pb_single.copy_area (0, 0, xpm_side, xpm_side, pb_out, xpm_side * (l * 4), xpm_side * k)
    # set pixbuf
    self.props.pixbuf = pb_out
