# -*- mode: python; python-indent: 2 -*-
#
# xbrightness-rgbgamma - X11 brightness / gamma adjustment tool
#
# Copyright (C) 2011 kakurasan
#
# Permission is hereby granted, free of charge, to any person obtaining a copy
# of this software and associated documentation files (the "Software"), to deal
# in the Software without restriction, including without limitation the rights
# to use, copy, modify, merge, publish, distribute, sublicense, and/or sell
# copies of the Software, and to permit persons to whom the Software is
# furnished to do so, subject to the following conditions:
#
# The above copyright notice and this permission notice shall be included in
# all copies or substantial portions of the Software.
#
# THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
# IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
# FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
# AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
# LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
# OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN
# THE SOFTWARE.
#
# THANKS:
#  xbrightness by Asher Blum and Jean-Daniel Pauget

import gtk


class GraphDrawingArea (gtk.DrawingArea):
  """
  DrawingArea for gamma curve
  """
  def __init__ (self, lib):
    gtk.DrawingArea.__init__ (self)
    self.connect ('size-allocate', self.__on_self_size_allocate)
    self.connect ('expose-event', self.__on_self_expose_event)
    self.__lib = lib
    self.__ramp_red = lib.ramp_red
    self.__ramp_green = lib.ramp_green
    self.__ramp_blue = lib.ramp_blue
    self.__ramp_size = lib.ramp_size
  def __on_self_size_allocate (self, widget, allocation):
    """
    'size-allocate' handler for this DrawingArea
    """
    self.__width = allocation.width
    self.__height = allocation.height
  def __on_self_expose_event (self, widget, event):
    """
    Draw gamma curve
    """
    x_step = float (self.__width) / float (self.__ramp_size - 1)
    y_step = self.__height / 256.0
    ctx_bg = widget.window.cairo_create ()
    ctx_bg.set_source_rgb (1.0, 1.0, 1.0)
    ctx_bg.rectangle (0, 0, self.__width, self.__height)
    ctx_bg.fill ()
    ctx_r = widget.window.cairo_create ()
    ctx_r.set_source_rgba (0.8, 0.0, 0.0, 0.5)
    ctx_r.new_path ()
    ctx_g = widget.window.cairo_create ()
    ctx_g.set_source_rgba (0.0, 0.8, 0.0, 0.5)
    ctx_g.new_path ()
    ctx_b = widget.window.cairo_create ()
    ctx_b.set_source_rgba (0.0, 0.0, 0.8, 0.5)
    ctx_b.new_path ()
    for i in range (self.__ramp_size):
      ctx_r.line_to (x_step * i, y_step * (65535 - self.__ramp_red[i]) / 256.0)
      ctx_g.line_to (x_step * i, y_step * (65535 - self.__ramp_green[i]) / 256.0)
      ctx_b.line_to (x_step * i, y_step * (65535 - self.__ramp_blue[i]) / 256.0)
    ctx_r.stroke ()
    ctx_g.stroke ()
    ctx_b.stroke ()
    return False
  def redraw (self):
    """
    Force redraw
    """
    self.__ramp_red = self.__lib.ramp_red
    self.__ramp_green = self.__lib.ramp_green
    self.__ramp_blue = self.__lib.ramp_blue
    self.window.invalidate_rect (gtk.gdk.Rectangle (0, 0, self.__width, self.__height), False)
    self.window.process_updates (False)
