# -*- mode: python; python-indent: 2 -*-
#
# xbrightness-rgbgamma - X11 brightness / gamma adjustment tool
#
# Copyright (C) 2011 kakurasan
#
# Permission is hereby granted, free of charge, to any person obtaining a copy
# of this software and associated documentation files (the "Software"), to deal
# in the Software without restriction, including without limitation the rights
# to use, copy, modify, merge, publish, distribute, sublicense, and/or sell
# copies of the Software, and to permit persons to whom the Software is
# furnished to do so, subject to the following conditions:
#
# The above copyright notice and this permission notice shall be included in
# all copies or substantial portions of the Software.
#
# THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
# IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
# FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
# AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
# LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
# OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN
# THE SOFTWARE.
#
# THANKS:
#  xbrightness by Asher Blum and Jean-Daniel Pauget

from xbrightness_rgbgamma.AppInfo import AppInfo
import xbrightness_rgbgamma.GuiFunctions as GuiFunctions
import gettext
import os


def main ():
  """
  Main
  """
  # NLS
  gettext.install (AppInfo.pkgname,
                   os.path.join (AppInfo.prefix, 'share', 'locale'),
                   names=('ngettext',))

  # check PyGTK
  try:
    import pygtk
    import glib
    import gtk
    pygtk.require ('2.0')
  except:
    return _('Error: ') + _('PyGTK (version 2) not installed.')

  # icon
  try:
    AppInfo.icon = gtk.gdk.pixbuf_new_from_file (os.path.join (AppInfo.prefix,
                                                               'share',
                                                               'pixmaps',
                                                               AppInfo.pkgname + '-gtk.png'))
  except:
    pass

  # load/check Libxxf86vm
  try:
    import xbrightness_rgbgamma.Libxxf86vm as Libxxf86vm
    from xbrightness_rgbgamma.GuiMainWindow import GuiMainWindow
  except ImportError as e:
    GuiFunctions.run_errordialog (None, _('Could not load libXxf86vm'), e)
    return _('Error: ') + _('Could not load libXxf86vm') + ': {0}'.format (e)
  try:
    lib = Libxxf86vm.Libxxf86vm ()
  except Libxxf86vm.XOpenDisplayError:
    try:
      return _('Error: ') + _('Could not open display: ') + '"{0}"'.format (os.environ['DISPLAY'])
    except KeyError:
      return _('Error: ') + _('Could not open display: ') + _('(undefined)')
  except Libxxf86vm.GetGammaRampSizeError:
    return _('Error: ') + _('Could not get size of gamma ramp.')
  except Libxxf86vm.GetGammaRampError:
    return _('Error: ') + _('Could not get gamma ramp.')

  # GUI
  win = GuiMainWindow (lib)
  win.show_all ()
  gtk.main ()

  return None
