# -*- mode: python; python-indent: 2 -*-
#
# xbrightness-rgbgamma - X11 brightness / gamma adjustment tool
#
# Copyright (C) 2011 kakurasan
#
# Permission is hereby granted, free of charge, to any person obtaining a copy
# of this software and associated documentation files (the "Software"), to deal
# in the Software without restriction, including without limitation the rights
# to use, copy, modify, merge, publish, distribute, sublicense, and/or sell
# copies of the Software, and to permit persons to whom the Software is
# furnished to do so, subject to the following conditions:
#
# The above copyright notice and this permission notice shall be included in
# all copies or substantial portions of the Software.
#
# THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
# IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
# FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
# AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
# LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
# OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN
# THE SOFTWARE.
#
# THANKS:
#  xbrightness by Asher Blum and Jean-Daniel Pauget

from xbrightness_rgbgamma.AppInfo import AppInfo
from xbrightness_rgbgamma.LabelWithUnderline import LabelWithUnderline
from xbrightness_rgbgamma.GraphDrawingArea import GraphDrawingArea
from xbrightness_rgbgamma.GammaChart import GammaChart
import xbrightness_rgbgamma.GuiFunctions as GuiFunctions
import xbrightness_rgbgamma.Libxxf86vm as Libxxf86vm
import glib
import gtk
import os


class GuiMainWindow (gtk.Window):
  """
  Application main window
  """
  __toid = None
  __desktopfile = os.path.join (glib.get_user_config_dir (), 'autostart', 'xbrightness-rgbgamma.desktop')
  def __init__ (self, lib):
    gtk.Window.__init__ (self)
    # lib
    self.__lib = lib
    self.__brightness = self.__lib.brightness
    self.__gamma_red = self.__lib.gamma_red
    self.__gamma_green = self.__lib.gamma_green
    self.__gamma_blue = self.__lib.gamma_blue
    # menu
    self.__accelgroup = gtk.AccelGroup ()
    self.add_accel_group (self.__accelgroup)
    self.__item_quit = gtk.ImageMenuItem (gtk.STOCK_QUIT, self.__accelgroup)
    self.__item_about = gtk.ImageMenuItem (gtk.STOCK_ABOUT, self.__accelgroup)
    self.__item_remove_autostart = gtk.CheckMenuItem (label=_('Remove _autostart file on exit'))
    self.__item_remove_autostart.props.use_underline = True
    self.__menu_file = gtk.Menu ()
    self.__menu_file.add (self.__item_remove_autostart)
    self.__menu_file.add (gtk.SeparatorMenuItem ())
    self.__menu_file.add (self.__item_quit)
    self.__menu_help = gtk.Menu ()
    self.__menu_help.add (self.__item_about)
    self.__item_file = gtk.MenuItem (_('_File'))
    self.__item_file.props.submenu = self.__menu_file
    self.__item_help = gtk.MenuItem (_('_Help'))
    self.__item_help.props.submenu = self.__menu_help
    self.__menubar = gtk.MenuBar ()
    self.__menubar.append (self.__item_file)
    self.__menubar.append (self.__item_help)
    # graph
    self.__drawingarea = GraphDrawingArea (self.__lib)
    # values
    self.__adjustment_brightness = gtk.Adjustment (value=self.__brightness, lower=0, upper=65535, step_incr=1, page_incr=100)
    self.__adjustment_gamma_red = gtk.Adjustment (value=self.__gamma_red, lower=0.01, upper=3, step_incr=0.01, page_incr=0.05)
    self.__adjustment_gamma_green = gtk.Adjustment (value=self.__gamma_green, lower=0.01, upper=3, step_incr=0.01, page_incr=0.05)
    self.__adjustment_gamma_blue = gtk.Adjustment (value=self.__gamma_blue, lower=0.01, upper=3, step_incr=0.01, page_incr=0.05)
    self.__hscale_brightness = gtk.HScale (self.__adjustment_brightness)
    self.__hscale_gamma_red = gtk.HScale (self.__adjustment_gamma_red)
    self.__hscale_gamma_green = gtk.HScale (self.__adjustment_gamma_green)
    self.__hscale_gamma_blue = gtk.HScale (self.__adjustment_gamma_blue)
    self.__hscale_brightness.props.draw_value \
     = self.__hscale_gamma_red.props.draw_value \
     = self.__hscale_gamma_green.props.draw_value \
     = self.__hscale_gamma_blue.props.draw_value = False
    self.__spbtn_brightness = gtk.SpinButton (self.__adjustment_brightness)
    self.__spbtn_gamma_red = gtk.SpinButton (self.__adjustment_gamma_red, digits=2)
    self.__spbtn_gamma_green = gtk.SpinButton (self.__adjustment_gamma_green, digits=2)
    self.__spbtn_gamma_blue = gtk.SpinButton (self.__adjustment_gamma_blue, digits=2)
    # frames
    self.__frame_brightness = gtk.Frame (_('Bright_ness'))
    self.__frame_brightness.props.label_widget.props.use_underline = True
    self.__frame_brightness.props.label_widget.props.mnemonic_widget = self.__spbtn_brightness
    self.__frame_gamma = gtk.Frame (_('Gamma correction'))
    self.__frame_gamma_red = gtk.Frame (_('_Red'))
    self.__frame_gamma_red.props.label_widget.props.use_underline = True
    self.__frame_gamma_red.props.label_widget.props.mnemonic_widget = self.__spbtn_gamma_red
    self.__frame_gamma_green = gtk.Frame (_('_Green'))
    self.__frame_gamma_green.props.label_widget.props.use_underline = True
    self.__frame_gamma_green.props.label_widget.props.mnemonic_widget = self.__spbtn_gamma_green
    self.__frame_gamma_blue = gtk.Frame (_('_Blue'))
    self.__frame_gamma_blue.props.label_widget.props.use_underline = True
    self.__frame_gamma_blue.props.label_widget.props.mnemonic_widget = self.__spbtn_gamma_blue
    self.__frame_brightness.props.shadow_type = self.__frame_gamma.props.shadow_type = gtk.SHADOW_IN
    # "apply" button
    self.__button_apply = gtk.Button (stock=gtk.STOCK_APPLY)
    self.__button_apply.props.sensitive = False
    # gamma chart
    self.__notebook_gamma = gtk.Notebook ()
    self.__notebook_gamma.append_page (GammaChart (gamma=2.2), LabelWithUnderline ('2._2'))
    self.__notebook_gamma.append_page (GammaChart (gamma=2.0), LabelWithUnderline ('2._0'))
    self.__notebook_gamma.append_page (GammaChart (gamma=1.8), LabelWithUnderline ('1._8'))
    self.__notebook_gamma.append_page (GammaChart (gamma=1.0), LabelWithUnderline ('_1.0'))
    # statusbar
    self.__statusbar = gtk.Statusbar ()
    self.__ctxid_status = self.__statusbar.get_context_id ('status')
    self.__ctxid_count = self.__statusbar.get_context_id ('count')
    # layout
    self.__aspectframe = gtk.AspectFrame (yalign=0)
    self.__aspectframe.add (self.__drawingarea)
    self.__notebook_graph_chart = gtk.Notebook ()
    self.__notebook_graph_chart.append_page (self.__aspectframe, LabelWithUnderline (_('Gra_ph')))
    self.__notebook_graph_chart.append_page (self.__notebook_gamma, LabelWithUnderline (_('Gamma _chart')))
    self.__hbox_brightness = gtk.HBox ()
    self.__hbox_brightness.pack_start (self.__hscale_brightness)
    self.__hbox_brightness.pack_start (self.__spbtn_brightness, expand=False, fill=False)
    self.__hbox_gamma_red = gtk.HBox ()
    self.__hbox_gamma_red.pack_start (self.__hscale_gamma_red)
    self.__hbox_gamma_red.pack_start (self.__spbtn_gamma_red, expand=False, fill=False)
    self.__hbox_gamma_green = gtk.HBox ()
    self.__hbox_gamma_green.pack_start (self.__hscale_gamma_green)
    self.__hbox_gamma_green.pack_start (self.__spbtn_gamma_green, expand=False, fill=False)
    self.__hbox_gamma_blue = gtk.HBox ()
    self.__hbox_gamma_blue.pack_start (self.__hscale_gamma_blue)
    self.__hbox_gamma_blue.pack_start (self.__spbtn_gamma_blue, expand=False, fill=False)
    self.__frame_gamma_red.add (self.__hbox_gamma_red)
    self.__frame_gamma_green.add (self.__hbox_gamma_green)
    self.__frame_gamma_blue.add (self.__hbox_gamma_blue)
    self.__vbox_gamma = gtk.VBox ()
    self.__vbox_gamma.pack_start (self.__frame_gamma_red)
    self.__vbox_gamma.pack_start (self.__frame_gamma_green)
    self.__vbox_gamma.pack_start (self.__frame_gamma_blue)
    self.__frame_brightness.add (self.__hbox_brightness)
    self.__frame_gamma.add (self.__vbox_gamma)
    self.__vbox_right = gtk.VBox ()
    self.__vbox_right.props.width_request = 200
    self.__vbox_right.pack_start (self.__frame_brightness)
    self.__vbox_right.pack_start (self.__frame_gamma)
    self.__vbox_right.pack_start (self.__button_apply, expand=False, fill=False)
    self.__hbox = gtk.HBox ()
    self.__hbox.pack_start (self.__notebook_graph_chart)
    self.__hbox.pack_start (self.__vbox_right, expand=False, fill=False)
    self.__vbox = gtk.VBox ()
    self.__vbox.pack_start (self.__menubar, expand=False, fill=False)
    self.__vbox.pack_start (self.__hbox)
    self.__vbox.pack_start (self.__statusbar, expand=False, fill=False)
    # signals
    self.__item_quit.connect ('activate', self.__on_quit)
    self.__item_about.connect ('activate', GuiFunctions.run_aboutdialog)
    self.__adjustment_brightness.connect ('value-changed', self.__on_adjustment_value_changed)
    self.__adjustment_gamma_red.connect ('value-changed', self.__on_adjustment_value_changed)
    self.__adjustment_gamma_green.connect ('value-changed', self.__on_adjustment_value_changed)
    self.__adjustment_gamma_blue.connect ('value-changed', self.__on_adjustment_value_changed)
    self.__button_apply.connect ('clicked', self.__on_button_apply_clicked)
    self.connect ('delete_event', self.__on_quit)
    # window
    self.props.title = AppInfo.name
    self.props.icon = AppInfo.icon
    self.props.width_request = 450
    self.add (self.__vbox)
  def __on_adjustment_value_changed (self, adjustment):
    """
    'value-changed' handler for brightness/gamma
    """
    try:
      self.__lib.set_gamma_and_brightness (self.__adjustment_brightness.props.value,
                                           self.__adjustment_gamma_red.props.value,
                                           self.__adjustment_gamma_green.props.value,
                                           self.__adjustment_gamma_blue.props.value)
      self.__drawingarea.redraw ()
      self.__statusbar.pop (self.__ctxid_status)
      if adjustment == self.__adjustment_brightness:
        self.__statusbar.push (self.__ctxid_status, _('Set brightness to {0:d}').format (int (adjustment.props.value)))
      elif adjustment == self.__adjustment_gamma_red:
        self.__statusbar.push (self.__ctxid_status, _('Set gamma/red to {0:.2f}').format (adjustment.props.value))
      elif adjustment == self.__adjustment_gamma_green:
        self.__statusbar.push (self.__ctxid_status, _('Set gamma/green to {0:.2f}').format (adjustment.props.value))
      elif adjustment == self.__adjustment_gamma_blue:
        self.__statusbar.push (self.__ctxid_status, _('Set gamma/blue to {0:.2f}').format (adjustment.props.value))
      if not self.__button_apply.props.sensitive:
        self.__button_apply.props.sensitive = True
        self.__cnt = 10
        self.__toid = glib.timeout_add_seconds (1, self.__to_revert_countdown)
    except Libxxf86vm.SetGammaRampError:
      GuiFunctions.run_errordialog (self, _('Could not set gamma ramp.'))
  def __to_revert_countdown (self):
    """
    Count down "cancel after N sec(s)" counter
    """
    self.__cnt -= 1
    self.__statusbar.pop (self.__ctxid_count)
    self.__statusbar.push (self.__ctxid_count, ngettext ('Cancel after {0} second', 'Cancel after {0} seconds', self.__cnt).format (self.__cnt))
    if self.__cnt > 0:
      return True  # continue
    else:
      # revert: lib
      try:
        self.__lib.set_gamma_and_brightness (self.__brightness,
                                             self.__gamma_red,
                                             self.__gamma_green,
                                             self.__gamma_blue)
        self.__drawingarea.redraw ()
        # revert: widgets
        self.__adjustment_brightness.props.value = self.__brightness
        self.__adjustment_gamma_red.props.value = self.__gamma_red
        self.__adjustment_gamma_green.props.value = self.__gamma_green
        self.__adjustment_gamma_blue.props.value = self.__gamma_blue
        self.__statusbar.pop (self.__ctxid_status)
        self.__statusbar.push (self.__ctxid_status, _('Values are reverted').format (self.__cnt))
        self.__button_apply.props.sensitive = False
      except Libxxf86vm.SetGammaRampError:
        GuiFunctions.run_errordialog (self, _('Could not set gamma ramp.'))
      self.__toid = None
      return False  # stop
  def __on_button_apply_clicked (self, widget):
    """
    Apply values
    """
    glib.source_remove (self.__toid)
    self.__toid = None
    self.__brightness = self.__adjustment_brightness.props.value
    self.__gamma_red = self.__adjustment_gamma_red.props.value
    self.__gamma_green = self.__adjustment_gamma_green.props.value
    self.__gamma_blue = self.__adjustment_gamma_blue.props.value
    self.__button_apply.props.sensitive = False
    self.__statusbar.pop (self.__ctxid_status)
    self.__statusbar.push (self.__ctxid_status, _('Values are applied').format (self.__cnt))
    # autostart
    if os.access (os.path.join (os.path.dirname (AppInfo.prefix), 'exec-cli.py'), os.X_OK):
      # source (not installed)
      cmd = os.path.join (os.path.dirname (AppInfo.prefix), 'exec-cli.py')
    else:
      # system (installed)
      cmd = 'xbrightness-rgbgamma'
    try:
      try:
        with open (self.__desktopfile, 'w') as f_out:
          try:
            f_out.write ('''[Desktop Entry]
Type=Application
Exec={cmd} {brightness} {gamma_red} {gamma_green} {gamma_blue}
Hidden=false
NoDisplay=false
X-GNOME-Autostart-enabled=true
Name=xbrightness-rgbgamma
Comment=Brightness and gamma
'''.format (cmd=cmd,
            brightness=int (self.__brightness),
            gamma_red=self.__gamma_red,
            gamma_green=self.__gamma_green,
            gamma_blue=self.__gamma_blue))
          except IOError as e:
            GuiFunctions.run_errordialog (self, _('Could not write to file "{0}"').format (self.__desktopfile), e)
      except IOError as e:
        GuiFunctions.run_errordialog (self, _('Could not open file "{0}"').format (self.__desktopfile), e)
    except AttributeError:  # glib.get_user_config_dir() not available in PyGObject 2.16.x
      pass
  def __on_quit (self, widget, event=None):
    """
    Quit
    """
    if self.__toid:
      # if not applied, revert
      try:
        self.__lib.set_gamma_and_brightness (self.__brightness,
                                             self.__gamma_red,
                                             self.__gamma_green,
                                             self.__gamma_blue)
      except Libxxf86vm.SetGammaRampError:
        pass
    if self.__item_remove_autostart.props.active == True:
      # remove .desktop file
      try:
        os.unlink (self.__desktopfile)
      except OSError as e:
        if e.errno != errno.ENOENT:
          GuiFunctions.run_errordialog (self, _('Could not remove file "{0}"').format (self.__desktopfile), e)
    gtk.main_quit ()
